#!/usr/bin/env python

import os
import sys

import stdiomask
from gppt import PixivAuth
from pixivpy3.aapi import AppPixivAPI

from .bookmarks import PixivBookmarksDownloader
from .followings import PixivFollowingsDownloader
from .pixiv_types import LoginFailed

SAVE_DIR = os.getenv("SAVE_DIR", os.path.join(os.path.expanduser("~"), "pbd"))
"""client.json
{
  'pixiv_id' : '<change this>',
  'password' : '<change this>'
}
"""


def interact(
    aapi: AppPixivAPI, f: PixivFollowingsDownloader, b: PixivBookmarksDownloader
) -> None:
    def getch() -> str:
        c = stdiomask.getch()
        print()
        return c

    my_info = aapi.user_detail(aapi.user_id)
    total_following_len = my_info["profile"]["total_follow_users"]
    total_bookmark_len = my_info["profile"]["total_illust_bookmarks_public"]
    print(
        "[?]: Download all works of following? "
        "({} artists) (n/y): ".format(total_following_len),
        flush=True,
        end="",
    )
    if getch() == "y":
        f.get_all_following_works()
        print("\033[K[+]: Finish!")
    print(
        "[?]: Download all bookmarked? "
        "({} works) (n/y): ".format(total_bookmark_len),
        flush=True,
        end="",
    )
    if getch() == "y":
        b.get_all_bookmarked_works()
        print("\033[K[+]: Finish!")


def _main() -> None:
    aapi, login_info = PixivAuth().auth()
    f = PixivFollowingsDownloader(aapi, login_info, SAVE_DIR)
    b = PixivBookmarksDownloader(aapi, login_info, SAVE_DIR)
    if "-y" in sys.argv:
        f.get_all_following_works()
        print("\033[K[+]: Finish!")
        b.get_all_bookmarked_works()
        print("\033[K[+]: Finish!")
    else:
        interact(aapi, f, b)


def main() -> None:
    try:
        _main()
    except (KeyError, LoginFailed):
        print("\n[!]: Request limit seem to be exceeded. " "Try again later.")
    except KeyboardInterrupt:
        print("\n[!]: SIGINT")
    finally:
        print("\x1b[?25h", end="")


if __name__ == "__main__":
    main()



import os
from typing import Any, List, Optional, cast

from pixivpy3.utils import JsonDict

from .base import PixivBaseDownloader
from .pixiv_types import NextFollowingsRequest, UserInfo


class PixivFollowingsDownloader(PixivBaseDownloader):
    def retrieve_following(self) -> List[UserInfo]:
        users: List[UserInfo] = []
        next_qs = cast(Optional[NextFollowingsRequest], {})
        my_info = self.aapi.user_detail(self.aapi.user_id)
        total = my_info["profile"]["total_follow_users"]
        while next_qs is not None:
            if "user_id" not in next_qs:
                res_json: JsonDict = self.aapi.user_following(
                    self.login_info["response"]["user"]["id"]
                )
            else:
                res_json = self.aapi.user_following(**next_qs)

            next_qs = self.aapi.parse_qs(res_json.next_url)
            now_retrieved_len = len(users)
            users.extend(
                self.extract_artist_info(
                    res_json.user_previews, total, now_retrieved_len
                )
            )
            self.rand_sleep(1.5)

        return users

    def extract_artist_info(
        self, user_previews: Any, following_total: int, retrieved: int
    ) -> List[Any]:
        users: List[Any] = []
        d_width = len(str(following_total))
        if user_previews is None:
            print("\n[!]Warning: artist info seems to be empty.")
            return users
        for idx, user in enumerate(user_previews):
            user_info: JsonDict = user.user
            print(
                f"\033[K[+]: [%0{d_width}d/%0{d_width}d]: %s (id: %d)"
                % (retrieved + idx + 1, following_total, user_info.name, user_info.id),
                end="\r",
                flush=True,
            )
            users.append(
                {
                    "id": user_info.id,
                    "name": user_info.name,
                    "account": user_info.account,
                    "illusts": self.retrieve_works(user_info.id),
                }
            )
            self.rand_sleep(1.5)
        else:
            return users

    def get_all_following_works(self) -> None:
        print("[+]: Fetching infomation of works of following artists...")
        following_data = self.retrieve_following()
        print("[+]: Downloading works of following artists...")
        following_len = len(following_data)
        d_width = len(str(following_len))
        for idx, author_data in enumerate(following_data):
            dirname = "{}_{}_{}".format(
                author_data["id"], author_data["name"], author_data["account"]
            ).replace("/", "／")
            print(
                f"\033[K[Artist][%0{d_width}d/%0{d_width}d]: %s"
                % (idx + 1, following_len, dirname)
            )
            self.download(
                author_data["illusts"],
                os.path.join(self.save_dir, "following", dirname),
            )
            print("\033[K\033[A\033[K", end="", flush=True)



import os
from typing import List, Optional, cast

from pixivpy3.utils import JsonDict

from .base import PixivBaseDownloader
from .pixiv_types import IllustInfo, NextBookmarksRequest


class PixivBookmarksDownloader(PixivBaseDownloader):
    def get_all_bookmarked_works(self) -> None:
        print("[+]: Fetching infomation of bookmarked works...")
        bookmarked_data = self.retrieve_bookmarks()
        print("\n[+]: Downloading bookmarked works...")
        self.download(bookmarked_data, os.path.join(self.save_dir, "bookmarks"))

    def retrieve_bookmarks(self) -> List[IllustInfo]:
        urls: List[IllustInfo] = []
        next_qs = cast(Optional[NextBookmarksRequest], {})
        target_id = self.login_info["response"]["user"]["id"]
        total = self.aapi.user_detail(self.aapi.user_id)["profile"][
            "total_illust_bookmarks_public"
        ]
        d_width = len(str(total))
        urls_len = 0
        while next_qs is not None:
            if "user_id" not in next_qs:
                res_json: JsonDict = self.aapi.user_bookmarks_illust(target_id)
            else:
                res_json = self.aapi.user_bookmarks_illust(**next_qs)
            for idx, illust in enumerate(res_json["illusts"]):
                print(
                    f"\033[K[+]: [%0{d_width}d/%0{d_width}d]: %s (id: %d)"
                    % (urls_len + idx + 1, total, illust.title, illust.id),
                    end="\r",
                    flush=True,
                )
                urls.append(
                    {
                        "id": illust.id,
                        "title": illust.title,
                        "link": self.ext_links(illust),
                    }
                )
            next_qs = self.aapi.parse_qs(res_json["next_url"])
            urls_len = len(urls)
            self.rand_sleep(0.5)
        else:
            return urls



import os
import random
import time
from typing import Dict, List, Optional, Union

from gppt import LoginInfo
from pixivpy3 import AppPixivAPI
from pixivpy3.utils import JsonDict

from .pixiv_types import IllustInfo


class PixivBaseDownloader:
    def __init__(self, aapi: AppPixivAPI, login_info: LoginInfo, save_dir: str):
        self.aapi = aapi
        self.login_info = login_info
        self.save_dir = save_dir

    @staticmethod
    def rand_sleep(base: float = 0.1, rand: float = 2.5) -> None:
        time.sleep(base + rand * random.random())

    @staticmethod
    def ext_links(illust: JsonDict) -> Union[List[str], str]:
        links: List[str] = [page.image_urls.original for page in illust.meta_pages]
        link: str = illust.meta_single_page.get(
            "original_image_url", illust.image_urls.large
        )

        return links if links != [] else link

    def retrieve_works(self, target_id: int) -> List[IllustInfo]:
        urls: List[IllustInfo] = []
        next: Optional[Dict[str, str]] = {}
        while next is not None:
            if next == {}:
                res_json = self.aapi.user_illusts(target_id, type="illust")
            else:
                res_json = self.aapi.user_illusts(**next)  # type: ignore
            if "error" in res_json and "invalid_grant" in res_json["error"]["message"]:
                self.aapi.auth()
                continue
            for illust in res_json["illusts"]:
                urls.append(
                    {
                        "id": illust.id,
                        "title": illust.title,
                        "link": self.ext_links(illust),
                    }
                )
            next = self.aapi.parse_qs(res_json["next_url"])
            self.rand_sleep(1.5)
        else:
            return urls

    def download(self, data: List[IllustInfo], save_path: str) -> None:
        os.makedirs(save_path, exist_ok=True)
        data_len = len(data)
        d_width = len(str(data_len))
        for idx, image_data in enumerate(data):
            title, id_ = image_data["title"].replace("/", "／"), image_data["id"]
            links = image_data["link"]
            print(
                f"\033[K[%0{d_width}d/%0{d_width}d]: %s (id: %d)"
                % (idx + 1, data_len, title, id_)
            )
            self.__download(links, title, id_, save_path)

            print("\033[K\033[A\033[K", end="", flush=True)

    def __download(
        self, links: Union[str, List[str]], title: str, id_: int, save_path: str
    ) -> None:
        if type(links) is list:
            for link in links:
                basename_: str = link.split("/")[-1]
                fname = "{}_{}_{}".format(id_, title, basename_.split("_")[-1])
                print("\033[K" + fname, end="\r")
                self.aapi.download(link, path=save_path, fname=fname)
        elif type(links) is str:
            link = links
            basename_ = link.split("/")[-1]
            fname = "{}_{}_{}".format(id_, title, basename_.split("_")[-1])
            print("\033[K" + fname, end="\r")
            self.aapi.download(link, path=save_path, fname=fname)



from .base import PixivBaseDownloader  # noqa: F401
from .bookmarks import PixivBookmarksDownloader  # noqa: F401
from .followings import PixivFollowingsDownloader  # noqa: F401
from .pixiv_types import (IllustInfo, LoginCred, LoginFailed,  # noqa: F401
                          NextBookmarksRequest, NextFollowingsRequest,
                          UserInfo)



from typing import List, TypedDict, Union


class LoginFailed(Exception):
    pass


class LoginCred(TypedDict):
    pixiv_id: str
    password: str


class IllustInfo(TypedDict):
    id: int
    title: str
    link: Union[str, List[str]]


class UserInfo(TypedDict):
    id: int
    name: str
    account: str
    illusts: List[IllustInfo]


class NextBookmarksRequest(TypedDict):
    user_id: str
    restrict: str
    filter: str
    max_bookmark_id: str


class NextFollowingsRequest(TypedDict):
    user_id: str
    restrict: str
    offset: str



from typing import List

from setuptools import find_packages, setup  # type: ignore

"""Update:
(change version,)
sudo rm -rf build dist *.egg-info
python setup.py sdist bdist_wheel
python -m twine upload --repository pypi dist/*
"""


def parse_requires():
    # type: () -> List[str]
    d = open("requirements.txt").read()
    return d.replace("\r", "").rstrip().split("\n")


setup(
    name="pixiv-bulk-downloader",
    version="2.2",
    description="Pixiv Bulk Downloader",
    description_content_type="",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/eggplants/pixiv-bulk-downloader",
    author="eggplants",
    packages=find_packages(),
    python_requires=">=3.9",
    include_package_data=True,
    license="MIT",
    install_requires=parse_requires(),
    entry_points={"console_scripts": ["pbd=pbd.main:main"]},
)
